<div class="image-content{% if content.position %} image-content-{{ content.position }}{% endif %}{% if content.format %} image-content-format-{{ content.format|slugify }} image-content-format-{{ content.get_format_display|slugify }}{% endif %}"><img src="{{ content.get_image.url }}" alt="{{ content.alt_text }}" />{% if content.caption %}<div class="caption">{{ content.caption }}</div>{% endif %}</div>
